@@ -1,55 +1,66 @@
 /* adapter_asis.c - adapt types as they are
  *
- * Copyright (C) 2003-2004 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2003-2010 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of psycopg.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2,
- * or (at your option) any later version.
+ * psycopg2 is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * In addition, as a special exception, the copyright holders give
+ * permission to link this program with the OpenSSL library (or with
+ * modified versions of OpenSSL that use the same license as OpenSSL),
+ * and distribute linked combinations including the two.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You must obey the GNU Lesser General Public License in all respects for
+ * all of the code used other than OpenSSL.
+ *
+ * psycopg2 is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
  */
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-#include <structmember.h>
-#include <stringobject.h>
-#include <string.h>
-
 #define PSYCOPG_MODULE
-#include "psycopg/config.h"
-#include "psycopg/python.h"
 #include "psycopg/psycopg.h"
+
 #include "psycopg/adapter_asis.h"
 #include "psycopg/microprotocols_proto.h"
 
+#include <string.h>
+
+
 /** the AsIs object **/
 
 static PyObject *
-asis_str(asisObject *self)
+asis_getquoted(asisObject *self, PyObject *args)
 {
+    PyObject *rv;
     if (self->wrapped == Py_None) {
-        return PyString_FromString("NULL");
+        Py_INCREF(psyco_null);
+        rv = psyco_null;
     }
     else {
-        return PyObject_Str(self->wrapped);
+        rv = PyObject_Str(self->wrapped);
+#if PY_MAJOR_VERSION > 2
+        /* unicode to bytes in Py3 */
+        if (rv) {
+            PyObject *tmp = PyUnicode_AsUTF8String(rv);
+            Py_DECREF(rv);
+            rv = tmp;
+        }
+#endif
     }
+
+    return rv;
 }
 
 static PyObject *
-asis_getquoted(asisObject *self, PyObject *args)
+asis_str(asisObject *self)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-    return asis_str(self);
+    return psycopg_ensure_text(asis_getquoted(self, NULL));
 }
 
 static PyObject *
@@ -73,14 +84,14 @@ asis_conform(asisObject *self, PyObject *args)
 /* object member list */
 
 static struct PyMemberDef asisObject_members[] = {
-    {"adapted", T_OBJECT, offsetof(asisObject, wrapped), RO},
+    {"adapted", T_OBJECT, offsetof(asisObject, wrapped), READONLY},
     {NULL}
 };
 
 /* object method table */
 
 static PyMethodDef asisObject_methods[] = {
-    {"getquoted", (PyCFunction)asis_getquoted, METH_VARARGS,
+    {"getquoted", (PyCFunction)asis_getquoted, METH_NOARGS,
      "getquoted() -> wrapped object value as SQL-quoted string"},
     {"__conform__", (PyCFunction)asis_conform, METH_VARARGS, NULL},
     {NULL}  /* Sentinel */
@@ -93,7 +104,7 @@ asis_setup(asisObject *self, PyObject *obj)
 {
     Dprintf("asis_setup: init asis object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
+        self, Py_REFCNT(self)
       );
 
     Py_INCREF(obj);
@@ -101,7 +112,7 @@ asis_setup(asisObject *self, PyObject *obj)
 
     Dprintf("asis_setup: good asis object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
+        self, Py_REFCNT(self)
       );
     return 0;
 }
@@ -122,10 +133,10 @@ asis_dealloc(PyObject* obj)
 
     Dprintf("asis_dealloc: deleted asis object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        obj, obj->ob_refcnt
+        obj, Py_REFCNT(obj)
       );
 
-    obj->ob_type->tp_free(obj);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 static int
@@ -164,8 +175,7 @@ asis_repr(asisObject *self)
 "AsIs(str) -> new AsIs adapter object"
 
 PyTypeObject asisType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "psycopg2._psycopg.AsIs",
     sizeof(asisObject),
     0,
@@ -237,5 +247,5 @@ psyco_AsIs(PyObject *module, PyObject *args)
     if (!PyArg_ParseTuple(args, "O", &obj))
         return NULL;
 
-    return PyObject_CallFunction((PyObject *)&asisType, "O", obj);
+    return PyObject_CallFunctionObjArgs((PyObject *)&asisType, obj, NULL);
 }
