@@ -23,90 +23,110 @@
  * License for more details.
  */
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-#include <structmember.h>
-#include <stringobject.h>
+#define PSYCOPG_MODULE
+#include "psycopg/psycopg.h"
+
+#include "psycopg/adapter_datetime.h"
+#include "psycopg/microprotocols_proto.h"
+
 #include <datetime.h>
 
 #include <time.h>
 #include <string.h>
 
-#define PSYCOPG_MODULE
-#include "psycopg/config.h"
-#include "psycopg/python.h"
-#include "psycopg/psycopg.h"
-#include "psycopg/adapter_datetime.h"
-#include "psycopg/microprotocols_proto.h"
-
-
-/* the pointer to the datetime module API is initialized by the module init
-   code, we just need to grab it */
-extern HIDDEN PyObject* pyDateTimeModuleP;
-extern HIDDEN PyObject *pyDateTypeP;
-extern HIDDEN PyObject *pyTimeTypeP;
-extern HIDDEN PyObject *pyDateTimeTypeP;
-extern HIDDEN PyObject *pyDeltaTypeP;
 
 extern HIDDEN PyObject *pyPsycopgTzModule;
 extern HIDDEN PyObject *pyPsycopgTzLOCAL;
 
+int
+psyco_adapter_datetime_init(void)
+{
+    Dprintf("psyco_adapter_datetime_init: datetime init");
+
+    PyDateTime_IMPORT;
+
+    if (!PyDateTimeAPI) {
+        PyErr_SetString(PyExc_ImportError, "datetime initialization failed");
+        return -1;
+    }
+    return 0;
+}
+
 /* datetime_str, datetime_getquoted - return result of quoting */
 
 static PyObject *
-pydatetime_str(pydatetimeObject *self)
+_pydatetime_string_date_time(pydatetimeObject *self)
 {
-    if (self->type <= PSYCO_DATETIME_TIMESTAMP) {
-        PyObject *tz;
+    PyObject *rv = NULL;
+    PyObject *iso = NULL;
+    PyObject *tz;
 
-        /* Select the right PG type to cast into. */
-        char *fmt = NULL;
-        switch (self->type) {
-        case PSYCO_DATETIME_TIME:
-            fmt = "'%s'::time";
-            break;
-        case PSYCO_DATETIME_DATE:
-            fmt = "'%s'::date";
-            break;
-        case PSYCO_DATETIME_TIMESTAMP:
-            tz = PyObject_GetAttrString(self->wrapped, "tzinfo");
-            if (!tz) { return NULL; }
-            fmt = (tz == Py_None) ? "'%s'::timestamp" : "'%s'::timestamptz";
-            Py_DECREF(tz);
-            break;
-        }
-
-        PyObject *res = NULL;
-        PyObject *iso = PyObject_CallMethod(self->wrapped, "isoformat", NULL);
-        if (iso) {
-            res = PyString_FromFormat(fmt, PyString_AsString(iso));
-            Py_DECREF(iso);
-        }
-        return res;
+    /* Select the right PG type to cast into. */
+    char *fmt = NULL;
+    switch (self->type) {
+    case PSYCO_DATETIME_TIME:
+        fmt = "'%s'::time";
+        break;
+    case PSYCO_DATETIME_DATE:
+        fmt = "'%s'::date";
+        break;
+    case PSYCO_DATETIME_TIMESTAMP:
+        tz = PyObject_GetAttrString(self->wrapped, "tzinfo");
+        if (!tz) { goto error; }
+        fmt = (tz == Py_None) ? "'%s'::timestamp" : "'%s'::timestamptz";
+        Py_DECREF(tz);
+        break;
     }
-    else {
-        PyDateTime_Delta *obj = (PyDateTime_Delta*)self->wrapped;
 
-        char buffer[8]; 
-        int i;
-        int a = obj->microseconds;
-
-        for (i=0; i < 6 ; i++) {
-            buffer[5-i] = '0' + (a % 10);
-            a /= 10;
-        }
-        buffer[6] = '\0';
-
-        return PyString_FromFormat("'%d days %d.%s seconds'::interval",
-                                   obj->days, obj->seconds, buffer);
+    if (!(iso = psycopg_ensure_bytes(
+            PyObject_CallMethod(self->wrapped, "isoformat", NULL)))) {
+        goto error;
     }
+
+    rv = Bytes_FromFormat(fmt, Bytes_AsString(iso));
+
+    Py_DECREF(iso);
+    return rv;
+
+error:
+    Py_XDECREF(iso);
+    return rv;
+}
+
+static PyObject *
+_pydatetime_string_delta(pydatetimeObject *self)
+{
+    PyDateTime_Delta *obj = (PyDateTime_Delta*)self->wrapped;
+
+    char buffer[8];
+    int i;
+    int a = obj->microseconds;
+
+    for (i=0; i < 6 ; i++) {
+        buffer[5-i] = '0' + (a % 10);
+        a /= 10;
+    }
+    buffer[6] = '\0';
+
+    return Bytes_FromFormat("'%d days %d.%s seconds'::interval",
+                            obj->days, obj->seconds, buffer);
 }
 
 static PyObject *
 pydatetime_getquoted(pydatetimeObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-    return pydatetime_str(self);
+    if (self->type <= PSYCO_DATETIME_TIMESTAMP) {
+        return _pydatetime_string_date_time(self);
+    }
+    else {
+        return _pydatetime_string_delta(self);
+    }
+}
+
+static PyObject *
+pydatetime_str(pydatetimeObject *self)
+{
+    return psycopg_ensure_text(pydatetime_getquoted(self, NULL));
 }
 
 static PyObject *
@@ -130,15 +150,15 @@ pydatetime_conform(pydatetimeObject *self, PyObject *args)
 /* object member list */
 
 static struct PyMemberDef pydatetimeObject_members[] = {
-    {"adapted", T_OBJECT, offsetof(pydatetimeObject, wrapped), RO},
-    {"type", T_INT, offsetof(pydatetimeObject, type), RO},
+    {"adapted", T_OBJECT, offsetof(pydatetimeObject, wrapped), READONLY},
+    {"type", T_INT, offsetof(pydatetimeObject, type), READONLY},
     {NULL}
 };
 
 /* object method table */
 
 static PyMethodDef pydatetimeObject_methods[] = {
-    {"getquoted", (PyCFunction)pydatetime_getquoted, METH_VARARGS,
+    {"getquoted", (PyCFunction)pydatetime_getquoted, METH_NOARGS,
      "getquoted() -> wrapped object value as SQL date/time"},
     {"__conform__", (PyCFunction)pydatetime_conform, METH_VARARGS, NULL},
     {NULL}  /* Sentinel */
@@ -151,7 +171,7 @@ pydatetime_setup(pydatetimeObject *self, PyObject *obj, int type)
 {
     Dprintf("pydatetime_setup: init datetime object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt);
+        self, Py_REFCNT(self));
 
     self->type = type;
     Py_INCREF(obj);
@@ -159,7 +179,7 @@ pydatetime_setup(pydatetimeObject *self, PyObject *obj, int type)
 
     Dprintf("pydatetime_setup: good pydatetime object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt);
+        self, Py_REFCNT(self));
     return 0;
 }
 
@@ -180,9 +200,9 @@ pydatetime_dealloc(PyObject* obj)
     Py_CLEAR(self->wrapped);
 
     Dprintf("mpydatetime_dealloc: deleted pydatetime object at %p, "
-            "refcnt = " FORMAT_CODE_PY_SSIZE_T, obj, obj->ob_refcnt);
+            "refcnt = " FORMAT_CODE_PY_SSIZE_T, obj, Py_REFCNT(obj));
 
-    obj->ob_type->tp_free(obj);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 static int
@@ -222,8 +242,7 @@ pydatetime_repr(pydatetimeObject *self)
 "datetime(datetime, type) -> new datetime wrapper object"
 
 PyTypeObject pydatetimeType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "psycopg2._psycopg.datetime",
     sizeof(pydatetimeObject),
     0,
@@ -298,7 +317,8 @@ psyco_Date(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "iii", &year, &month, &day))
         return NULL;
 
-    obj = PyObject_CallFunction(pyDateTypeP, "iii", year, month, day);
+    obj = PyObject_CallFunction((PyObject*)PyDateTimeAPI->DateType,
+                                "iii", year, month, day);
 
     if (obj) {
         res = PyObject_CallFunction((PyObject *)&pydatetimeType,
@@ -327,10 +347,10 @@ psyco_Time(PyObject *self, PyObject *args)
     second = floor(second);
 
     if (tzinfo == NULL)
-       obj = PyObject_CallFunction(pyTimeTypeP, "iiii",
+       obj = PyObject_CallFunction((PyObject*)PyDateTimeAPI->TimeType, "iiii",
             hours, minutes, (int)second, (int)round(micro));
     else
-       obj = PyObject_CallFunction(pyTimeTypeP, "iiiiO",
+       obj = PyObject_CallFunction((PyObject*)PyDateTimeAPI->TimeType, "iiiiO",
             hours, minutes, (int)second, (int)round(micro), tzinfo);
 
     if (obj) {
@@ -342,30 +362,25 @@ psyco_Time(PyObject *self, PyObject *args)
     return res;
 }
 
-PyObject *
-psyco_Timestamp(PyObject *self, PyObject *args)
+static PyObject *
+_psyco_Timestamp(int year, int month, int day,
+                 int hour, int minute, double second, PyObject *tzinfo)
 {
+    double micro;
+    PyObject *obj;
     PyObject *res = NULL;
-    PyObject *tzinfo = NULL;
-    int year, month, day;
-    int hour=0, minute=0; /* default to midnight */
-    double micro, second=0.0;
-
-    PyObject* obj = NULL;
-
-    if (!PyArg_ParseTuple(args, "lii|iidO", &year, &month, &day,
-                          &hour, &minute, &second, &tzinfo))
-        return NULL;
 
     micro = (second - floor(second)) * 1000000.0;
     second = floor(second);
 
     if (tzinfo == NULL)
-        obj = PyObject_CallFunction(pyDateTimeTypeP, "iiiiiii",
+        obj = PyObject_CallFunction((PyObject*)PyDateTimeAPI->DateTimeType,
+            "iiiiiii",
             year, month, day, hour, minute, (int)second,
             (int)round(micro));
     else
-        obj = PyObject_CallFunction(pyDateTimeTypeP, "iiiiiiiO",
+        obj = PyObject_CallFunction((PyObject*)PyDateTimeAPI->DateTimeType,
+            "iiiiiiiO",
             year, month, day, hour, minute, (int)second,
             (int)round(micro), tzinfo);
 
@@ -378,6 +393,21 @@ psyco_Timestamp(PyObject *self, PyObject *args)
     return res;
 }
 
+PyObject *
+psyco_Timestamp(PyObject *self, PyObject *args)
+{
+    PyObject *tzinfo = NULL;
+    int year, month, day;
+    int hour=0, minute=0; /* default to midnight */
+    double second=0.0;
+
+    if (!PyArg_ParseTuple(args, "iii|iidO", &year, &month, &day,
+                          &hour, &minute, &second, &tzinfo))
+        return NULL;
+
+    return _psyco_Timestamp(year, month, day, hour, minute, second, tzinfo);
+}
+
 PyObject *
 psyco_DateFromTicks(PyObject *self, PyObject *args)
 {
@@ -397,6 +427,10 @@ psyco_DateFromTicks(PyObject *self, PyObject *args)
             Py_DECREF(args);
         }
     }
+	else {
+		PyErr_SetString(InterfaceError, "failed localtime call");
+	}
+
     return res;
 }
 
@@ -421,6 +455,10 @@ psyco_TimeFromTicks(PyObject *self, PyObject *args)
             Py_DECREF(args);
         }
     }
+	else {
+		PyErr_SetString(InterfaceError, "failed localtime call");
+	}
+
     return res;
 }
 
@@ -438,20 +476,15 @@ psyco_TimestampFromTicks(PyObject *self, PyObject *args)
     t = (time_t)floor(ticks);
     ticks -= (double)t;
     if (localtime_r(&t, &tm)) {
-        PyObject *value = Py_BuildValue("iiiiidO",
-            tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday,
-            tm.tm_hour, tm.tm_min,
-            (double)tm.tm_sec + ticks,
+        res = _psyco_Timestamp(
+            tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
+            tm.tm_hour, tm.tm_min, (double)tm.tm_sec + ticks,
             pyPsycopgTzLOCAL);
-        if (value) {
-            /* FIXME: not decref'ing the value here is a memory leak
-	       but, on the other hand, if we decref we get a clean nice
-	       segfault (on my 64 bit Python 2.4 box). So this leaks
-	       will stay until after 2.0.7 when we'll try to plug it */
-            res = psyco_Timestamp(self, value);
-        }
     }
-    
+	else {
+		PyErr_SetString(InterfaceError, "failed localtime call");
+	}
+
     return res;
 }
 
@@ -462,7 +495,7 @@ psyco_DateFromPy(PyObject *self, PyObject *args)
 {
     PyObject *obj;
 
-    if (!PyArg_ParseTuple(args, "O!", pyDateTypeP, &obj))
+    if (!PyArg_ParseTuple(args, "O!", PyDateTimeAPI->DateType, &obj))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&pydatetimeType, "Oi", obj,
@@ -474,7 +507,7 @@ psyco_TimeFromPy(PyObject *self, PyObject *args)
 {
     PyObject *obj;
 
-    if (!PyArg_ParseTuple(args, "O!", pyTimeTypeP, &obj))
+    if (!PyArg_ParseTuple(args, "O!", PyDateTimeAPI->TimeType, &obj))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&pydatetimeType, "Oi", obj,
@@ -486,7 +519,7 @@ psyco_TimestampFromPy(PyObject *self, PyObject *args)
 {
     PyObject *obj;
 
-    if (!PyArg_ParseTuple(args, "O!", pyDateTimeTypeP, &obj))
+    if (!PyArg_ParseTuple(args, "O!", PyDateTimeAPI->DateTimeType, &obj))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&pydatetimeType, "Oi", obj,
@@ -498,7 +531,7 @@ psyco_IntervalFromPy(PyObject *self, PyObject *args)
 {
     PyObject *obj;
 
-    if (!PyArg_ParseTuple(args, "O!", pyDeltaTypeP, &obj))
+    if (!PyArg_ParseTuple(args, "O!", PyDateTimeAPI->DeltaType, &obj))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&pydatetimeType, "Oi", obj,
