@@ -27,7 +27,7 @@
 #define PSYCOPG_CONFIG_H 1
 
 /* GCC 4.0 and later have support for specifying symbol visibility */
-#if __GNUC__ >= 4
+#if __GNUC__ >= 4 && !defined(__MINGW32__)
 #  define HIDDEN __attribute__((visibility("hidden")))
 #else
 #  define HIDDEN
@@ -51,6 +51,10 @@ extern HIDDEN int psycopg_debug_enabled;
 #else /* !__GNUC__ or __APPLE__ */
 #ifdef PSYCOPG_DEBUG
 #include <stdarg.h>
+#ifdef _WIN32
+#include <process.h>
+#define getpid _getpid
+#endif
 static void Dprintf(const char *fmt, ...)
 {
     va_list ap;
@@ -119,16 +123,9 @@ static int pthread_mutex_init(pthread_mutex_t *mutex, void* fake)
 /* to work around the fact that Windows does not have a gmtime_r function, or
    a proper gmtime function */
 #ifdef _WIN32
-static struct tm *gmtime_r(time_t *t, struct tm *tm)
-{
-  tm = gmtime(t);
-  return tm;
-}
-static struct tm *localtime_r(time_t *t, struct tm *tm)
-{
-  tm = localtime(t);
-  return tm;
-}
+#define gmtime_r(t, tm) (gmtime(t)?memcpy((tm), gmtime(t), sizeof(*(tm))):NULL)
+#define localtime_r(t, tm) (localtime(t)?memcpy((tm), localtime(t), sizeof(*(tm))):NULL)
+
 /* remove the inline keyword, since it doesn't work unless C++ file */
 #define inline
 
@@ -139,6 +136,8 @@ static struct tm *localtime_r(time_t *t, struct tm *tm)
  * in libxml2 code */
 #define isinf(x) ((_fpclass(x) == _FPCLASS_PINF) ? 1 \
 	: ((_fpclass(x) == _FPCLASS_NINF) ? -1 : 0))
+
+#define strcasecmp(x, y) lstrcmpi(x, y)
 #endif
 #endif
 
@@ -161,4 +160,33 @@ static double round(double num)
 #define isinf(x) (!finite((x)) && (x)==(x))
 #endif
 
+/* decorators for the gcc cpychecker plugin */
+#if defined(WITH_CPYCHECKER_RETURNS_BORROWED_REF_ATTRIBUTE)
+#define BORROWED \
+    __attribute__((cpychecker_returns_borrowed_ref))
+#else
+#define BORROWED
+#endif
+
+#if defined(WITH_CPYCHECKER_STEALS_REFERENCE_TO_ARG_ATTRIBUTE)
+#define STEALS(n) \
+    __attribute__((cpychecker_steals_reference_to_arg(n)))
+#else
+#define STEALS(n)
+#endif
+
+#if defined(WITH_CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION_ATTRIBUTE)
+#define RAISES_NEG \
+    __attribute__((cpychecker_negative_result_sets_exception))
+#else
+#define RAISES_NEG
+#endif
+
+#if defined(WITH_CPYCHECKER_SETS_EXCEPTION_ATTRIBUTE)
+#define RAISES \
+    __attribute__((cpychecker_sets_exception))
+#else
+#define RAISES
+#endif
+
 #endif /* !defined(PSYCOPG_CONFIG_H) */
