@@ -23,17 +23,15 @@
  * License for more details.
  */
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-#include <string.h>
-
 #define PSYCOPG_MODULE
-#include "psycopg/config.h"
 #include "psycopg/psycopg.h"
-#include "psycopg/connection.h"
+
 #include "psycopg/lobject.h"
+#include "psycopg/connection.h"
 #include "psycopg/pqpath.h"
 
+#include <string.h>
+
 #ifdef PSYCOPG_EXTENSIONS
 
 static void
@@ -45,31 +43,145 @@ collect_error(connectionObject *conn, char **error)
         *error = strdup(msg);
 }
 
+
+/* Check if the mode passed to the large object is valid.
+ * In case of success return a value >= 0
+ * On error return a value < 0 and set an exception.
+ *
+ * Valid mode are [r|w|rw|n][t|b]
+ */
+RAISES_NEG static int
+_lobject_parse_mode(const char *mode)
+{
+    int rv = 0;
+    size_t pos = 0;
+
+    if (0 == strncmp("rw", mode, 2)) {
+        rv |= LOBJECT_READ | LOBJECT_WRITE;
+        pos += 2;
+    }
+    else {
+        switch (mode[0]) {
+        case 'r':
+            rv |= LOBJECT_READ;
+            pos += 1;
+            break;
+        case 'w':
+            rv |= LOBJECT_WRITE;
+            pos += 1;
+            break;
+        case 'n':
+            pos += 1;
+            break;
+        default:
+            rv |= LOBJECT_READ;
+            break;
+        }
+    }
+
+    switch (mode[pos]) {
+        case 't':
+            rv |= LOBJECT_TEXT;
+            pos += 1;
+            break;
+        case 'b':
+            rv |= LOBJECT_BINARY;
+            pos += 1;
+            break;
+        default:
+#if PY_MAJOR_VERSION < 3
+            rv |= LOBJECT_BINARY;
+#else
+            rv |= LOBJECT_TEXT;
+#endif
+            break;
+    }
+
+    if (pos != strlen(mode)) {
+        PyErr_Format(PyExc_ValueError,
+            "bad mode for lobject: '%s'", mode);
+        rv = -1;
+    }
+
+    return rv;
+}
+
+
+/* Return a string representing the lobject mode.
+ *
+ * The return value is a new string allocated on the Python heap.
+ *
+ * The function must be called holding the GIL.
+ */
+static char *
+_lobject_unparse_mode(int mode)
+{
+    char *buf;
+    char *c;
+
+    /* the longest is 'rwt' */
+    if (!(c = buf = PyMem_Malloc(4))) {
+        PyErr_NoMemory();
+        return NULL;
+    }
+
+    if (mode & LOBJECT_READ) { *c++ = 'r'; }
+    if (mode & LOBJECT_WRITE) { *c++ = 'w'; }
+
+    if (buf == c) {
+        /* neither read nor write */
+        *c++ = 'n';
+    }
+    else {
+        if (mode & LOBJECT_TEXT) {
+            *c++ = 't';
+        }
+        else {
+            *c++ = 'b';
+        }
+    }
+    *c = '\0';
+
+    return buf;
+}
+
 /* lobject_open - create a new/open an existing lo */
 
-int
+RAISES_NEG int
 lobject_open(lobjectObject *self, connectionObject *conn,
-              Oid oid, int mode, Oid new_oid, const char *new_file)
+              Oid oid, const char *smode, Oid new_oid, const char *new_file)
 {
     int retvalue = -1;
     PGresult *pgres = NULL;
     char *error = NULL;
+    int pgmode = 0;
+    int mode;
+
+    if (0 > (mode = _lobject_parse_mode(smode))) {
+        return -1;
+    }
 
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&(self->conn->lock));
 
-    retvalue = pq_begin_locked(self->conn, &pgres, &error);
+    retvalue = pq_begin_locked(self->conn, &pgres, &error, &_save);
     if (retvalue < 0)
         goto end;
 
     /* if the oid is InvalidOid we create a new lob before opening it
        or we import a file from the FS, depending on the value of
-       new_name */
+       new_file */
     if (oid == InvalidOid) {
         if (new_file)
             self->oid = lo_import(self->conn->pgconn, new_file);
-        else
-            self->oid = lo_create(self->conn->pgconn, new_oid);
+        else {
+            /* Use lo_creat when possible to be more middleware-friendly.
+               See ticket #88. */
+            if (new_oid != InvalidOid)
+                self->oid = lo_create(self->conn->pgconn, new_oid);
+            else
+                self->oid = lo_creat(self->conn->pgconn, INV_READ | INV_WRITE);
+        }
 
         Dprintf("lobject_open: large object created with oid = %d",
                 self->oid);
@@ -80,19 +192,19 @@ lobject_open(lobjectObject *self, connectionObject *conn,
             goto end;
         }
 
-        mode = INV_WRITE;
+        mode = (mode & ~LOBJECT_READ) | LOBJECT_WRITE;
     }
     else {
         self->oid = oid;
-        if (mode == 0) mode = INV_READ;
     }
 
-    /* if the oid is a real one we try to open with the given mode,
-       unless the mode is -1, meaning "don't open!" */
-    if (mode != -1) {
-        self->fd = lo_open(self->conn->pgconn, self->oid, mode);
-        Dprintf("lobject_open: large object opened with fd = %d",
-            self->fd);
+    /* if the oid is a real one we try to open with the given mode */
+    if (mode & LOBJECT_READ) { pgmode |= INV_READ; }
+    if (mode & LOBJECT_WRITE) { pgmode |= INV_WRITE; }
+    if (pgmode) {
+        self->fd = lo_open(self->conn->pgconn, self->oid, pgmode);
+        Dprintf("lobject_open: large object opened with mode = %i fd = %d",
+            pgmode, self->fd);
 
         if (self->fd == -1) {
             collect_error(self->conn, &error);
@@ -100,17 +212,17 @@ lobject_open(lobjectObject *self, connectionObject *conn,
             goto end;
         }
     }
+
     /* set the mode for future reference */
-    switch (mode) {
-    case -1:
-        self->smode = "n"; break;
-    case INV_READ:
-        self->smode = "r"; break;
-    case INV_WRITE:
-        self->smode = "w"; break;
-    case INV_READ+INV_WRITE:
-        self->smode = "rw"; break;
+    self->mode = mode;
+    Py_BLOCK_THREADS;
+    self->smode = _lobject_unparse_mode(mode);
+    Py_UNBLOCK_THREADS;
+    if (NULL == self->smode) {
+        retvalue = 1;  /* exception already set */
+        goto end;
     }
+
     retvalue = 0;
 
  end:
@@ -119,17 +231,34 @@ lobject_open(lobjectObject *self, connectionObject *conn,
 
     if (retvalue < 0)
         pq_complete_error(self->conn, &pgres, &error);
+    /* if retvalue > 0, an exception is already set */
+
     return retvalue;
 }
 
 /* lobject_close - close an existing lo */
 
-static int
+RAISES_NEG static int
 lobject_close_locked(lobjectObject *self, char **error)
 {
     int retvalue;
 
-    if (self->conn->isolation_level == 0 ||
+    Dprintf("lobject_close_locked: conn->closed %ld", self->conn->closed);
+    switch (self->conn->closed) {
+    case 0:
+        /* Connection is open, go ahead */
+        break;
+    case 1:
+        /* Connection is closed, return a success */
+        return 0;
+        break;
+    default:
+        PyErr_SetString(OperationalError, "the connection is broken");
+        return -1;
+        break;
+    }
+
+    if (self->conn->autocommit ||
         self->conn->mark != self->mark ||
         self->fd == -1)
         return 0;
@@ -142,7 +271,7 @@ lobject_close_locked(lobjectObject *self, char **error)
     return retvalue;
 }
 
-int
+RAISES_NEG int
 lobject_close(lobjectObject *self)
 {
     PGresult *pgres = NULL;
@@ -164,7 +293,7 @@ lobject_close(lobjectObject *self)
 
 /* lobject_unlink - remove an lo from database */
 
-int
+RAISES_NEG int
 lobject_unlink(lobjectObject *self)
 {
     PGresult *pgres = NULL;
@@ -174,7 +303,7 @@ lobject_unlink(lobjectObject *self)
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&(self->conn->lock));
 
-    retvalue = pq_begin_locked(self->conn, &pgres, &error);
+    retvalue = pq_begin_locked(self->conn, &pgres, &error, &_save);
     if (retvalue < 0)
         goto end;
 
@@ -198,24 +327,22 @@ lobject_unlink(lobjectObject *self)
 
 /* lobject_write - write bytes to a lo */
 
-Py_ssize_t
+RAISES_NEG Py_ssize_t
 lobject_write(lobjectObject *self, const char *buf, size_t len)
 {
     Py_ssize_t written;
     PGresult *pgres = NULL;
     char *error = NULL;
 
-    Dprintf("lobject_writing: fd = %d, len = %d",
+    Dprintf("lobject_writing: fd = %d, len = " FORMAT_CODE_SIZE_T,
             self->fd, len);
 
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&(self->conn->lock));
 
-    PQsetnonblocking(self->conn->pgconn, 0);
     written = lo_write(self->conn->pgconn, self->fd, buf, len);
     if (written < 0)
         collect_error(self->conn, &error);
-    PQsetnonblocking(self->conn->pgconn, 1);
 
     pthread_mutex_unlock(&(self->conn->lock));
     Py_END_ALLOW_THREADS;
@@ -227,7 +354,7 @@ lobject_write(lobjectObject *self, const char *buf, size_t len)
 
 /* lobject_read - read bytes from a lo */
 
-Py_ssize_t
+RAISES_NEG Py_ssize_t
 lobject_read(lobjectObject *self, char *buf, size_t len)
 {
     Py_ssize_t n_read;
@@ -251,7 +378,7 @@ lobject_read(lobjectObject *self, char *buf, size_t len)
 
 /* lobject_seek - move the current position in the lo */
 
-int
+RAISES_NEG int
 lobject_seek(lobjectObject *self, int pos, int whence)
 {
     PGresult *pgres = NULL;
@@ -279,7 +406,7 @@ lobject_seek(lobjectObject *self, int pos, int whence)
 
 /* lobject_tell - tell the current position in the lo */
 
-int
+RAISES_NEG int
 lobject_tell(lobjectObject *self)
 {
     PGresult *pgres = NULL;
@@ -306,7 +433,7 @@ lobject_tell(lobjectObject *self)
 
 /* lobject_export - export to a local file */
 
-int
+RAISES_NEG int
 lobject_export(lobjectObject *self, const char *filename)
 {
     PGresult *pgres = NULL;
@@ -316,7 +443,7 @@ lobject_export(lobjectObject *self, const char *filename)
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&(self->conn->lock));
 
-    retvalue = pq_begin_locked(self->conn, &pgres, &error);
+    retvalue = pq_begin_locked(self->conn, &pgres, &error, &_save);
     if (retvalue < 0)
         goto end;
 
@@ -333,6 +460,36 @@ lobject_export(lobjectObject *self, const char *filename)
     return retvalue;
 }
 
+#if PG_VERSION_HEX >= 0x080300
+
+RAISES_NEG int
+lobject_truncate(lobjectObject *self, size_t len)
+{
+    int retvalue;
+    PGresult *pgres = NULL;
+    char *error = NULL;
+
+    Dprintf("lobject_truncate: fd = %d, len = " FORMAT_CODE_SIZE_T,
+            self->fd, len);
+
+    Py_BEGIN_ALLOW_THREADS;
+    pthread_mutex_lock(&(self->conn->lock));
+
+    retvalue = lo_truncate(self->conn->pgconn, self->fd, len);
+    Dprintf("lobject_truncate: result = %d", retvalue);
+    if (retvalue < 0)
+        collect_error(self->conn, &error);
+
+    pthread_mutex_unlock(&(self->conn->lock));
+    Py_END_ALLOW_THREADS;
+
+    if (retvalue < 0)
+        pq_complete_error(self->conn, &pgres, &error);
+    return retvalue;
+
+}
+
+#endif /* PG_VERSION_HEX >= 0x080300 */
 
 #endif
 
