@@ -1,37 +1,35 @@
 /* microprotocol_proto.c - psycopg protocols
  *
- * Copyright (C) 2003 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2003-2010 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of psycopg.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2,
- * or (at your option) any later version.
+ * psycopg2 is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * In addition, as a special exception, the copyright holders give
+ * permission to link this program with the OpenSSL library (or with
+ * modified versions of OpenSSL that use the same license as OpenSSL),
+ * and distribute linked combinations including the two.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You must obey the GNU Lesser General Public License in all respects for
+ * all of the code used other than OpenSSL.
+ *
+ * psycopg2 is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
  */
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-#include <structmember.h>
-#include <stringobject.h>
+#define PSYCOPG_MODULE
+#include "psycopg/psycopg.h"
+
+#include "psycopg/microprotocols_proto.h"
 
 #include <string.h>
 
-#define PSYCOPG_MODULE
-#include "psycopg/config.h"
-#include "psycopg/python.h"
-#include "psycopg/psycopg.h"
-#include "psycopg/microprotocols_proto.h"
-
 
 /** void protocol implementation **/
 
@@ -44,8 +42,6 @@
 static PyObject *
 psyco_isqlquote_getquoted(isqlquoteObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -58,8 +54,6 @@ psyco_isqlquote_getquoted(isqlquoteObject *self, PyObject *args)
 static PyObject *
 psyco_isqlquote_getbinary(isqlquoteObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -72,8 +66,6 @@ psyco_isqlquote_getbinary(isqlquoteObject *self, PyObject *args)
 static PyObject *
 psyco_isqlquote_getbuffer(isqlquoteObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -87,11 +79,11 @@ psyco_isqlquote_getbuffer(isqlquoteObject *self, PyObject *args)
 
 static struct PyMethodDef isqlquoteObject_methods[] = {
     {"getquoted", (PyCFunction)psyco_isqlquote_getquoted,
-     METH_VARARGS, psyco_isqlquote_getquoted_doc},
+     METH_NOARGS, psyco_isqlquote_getquoted_doc},
     {"getbinary", (PyCFunction)psyco_isqlquote_getbinary,
-     METH_VARARGS, psyco_isqlquote_getbinary_doc},
+     METH_NOARGS, psyco_isqlquote_getbinary_doc},
     {"getbuffer", (PyCFunction)psyco_isqlquote_getbuffer,
-     METH_VARARGS, psyco_isqlquote_getbuffer_doc},
+     METH_NOARGS, psyco_isqlquote_getbuffer_doc},
     /*    {"prepare", (PyCFunction)psyco_isqlquote_prepare,
           METH_VARARGS, psyco_isqlquote_prepare_doc}, */
     {NULL}
@@ -101,7 +93,7 @@ static struct PyMethodDef isqlquoteObject_methods[] = {
 
 static struct PyMemberDef isqlquoteObject_members[] = {
     /* DBAPI-2.0 extensions (exception objects) */
-    {"_wrapped", T_OBJECT, offsetof(isqlquoteObject, wrapped), RO},
+    {"_wrapped", T_OBJECT, offsetof(isqlquoteObject, wrapped), READONLY},
     {NULL}
 };
 
@@ -123,7 +115,7 @@ isqlquote_dealloc(PyObject* obj)
 
     Py_XDECREF(self->wrapped);
 
-    obj->ob_type->tp_free(obj);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 static int
@@ -158,8 +150,7 @@ isqlquote_del(PyObject* self)
 "returning the SQL representation of the object.\n\n"
 
 PyTypeObject isqlquoteType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "psycopg2._psycopg.ISQLQuote",
     sizeof(isqlquoteObject),
     0,
