@@ -26,7 +26,7 @@
 #include <math.h>
 #include "datetime.h"
 
-int
+RAISES_NEG static int
 psyco_typecast_datetime_init(void)
 {
     Dprintf("psyco_typecast_datetime_init: datetime init");
@@ -159,7 +159,7 @@ typecast_PYDATETIME_cast(const char *str, Py_ssize_t len, PyObject *curs)
                 y, m, d, hh, mm, ss, us, tzinfo);
             Dprintf("typecast_PYDATETIME_cast: tzinfo: %p, refcnt = "
                 FORMAT_CODE_PY_SSIZE_T,
-                tzinfo, tzinfo->ob_refcnt
+                tzinfo, Py_REFCNT(tzinfo)
               );
             Py_DECREF(tzinfo);
         }
@@ -239,7 +239,7 @@ typecast_PYINTERVAL_cast(const char *str, Py_ssize_t len, PyObject *curs)
 
         case '0': case '1': case '2': case '3': case '4':
         case '5': case '6': case '7': case '8': case '9':
-            v = v*10 + (double)*str - (double)'0';
+            v = v * 10.0 + (double)(*str - '0');
             if (part == 6){
                 denominator *= 10;
             }
